-------------------------------------------------------------------------
--
--  graphic_memory_engine.vhd
--
--	Author 	: Rene Bllhoff
--  Create 	: Sep. 2006
--
--  Purpose : 
--
--	Hauptkomponente der Graphic Memory Engine
--	Stellt die Addressgenerierung, die Zhler, die Steuerung
--  durch die Statemachine, die Verbindung der Register (via SPI) mit den
--  internen Zhlern,Address-Basisregistern, Ausgaberegistern und Flags
--
--
--  (c)2005-2006 by Rene Bllhoff
-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity graphic_memory_engine is
    Port ( 	clk   			: in std_logic;
          	res 				: in std_logic;

						channel			: out std_logic;
						videooff		: out std_logic;
						videostart  : OUT std_logic_vector (19 downto 0);
						pal_idx			: out std_logic_vector ( 1 downto 0);

					  mpos_x  		: out std_logic_vector ( 9 downto 0);
					  mpos_y  		: out	std_logic_vector ( 9 downto 0);
					  mouseidx		: out	std_logic_vector ( 2 downto 0);
						mouse_on	  : out std_logic;

						gme_din 		: IN std_logic_vector(7 downto 0);
						gme_rdy 		: IN std_logic;
						gme_req 		: IN std_logic; 
						gme_adr 		: OUT std_logic_vector(19 downto 0);
						gme_dout 		: OUT std_logic_vector(7 downto 0);
						gme_rw 			: OUT std_logic;
						gme_trg 		: OUT std_logic;

						debug   			: out		std_logic_vector (7 downto 0);

						iomem_adr 	: in std_logic_vector(7 downto 0);
						iomem_rdata : out std_logic_vector(7 downto 0);   
						iomem_wdata : in std_logic_vector(7 downto 0);
						iomem_wen 	: in std_logic;
						iomem_en 		: in std_logic				   

				 );
end graphic_memory_engine;

architecture Behavioral of graphic_memory_engine is


	type gme_states is (	Idle, trg_reset,

												fill_setup_01, fill_setup_02,
												fill_write_01, fill_write_02, fill_write_03, fill_write_04,
												fill_inc_01, fill_inc_02, fill_inc_03,

												modify_setup_01, modify_setup_02,
												modify_read_01, modify_read_02, modify_read_03, modify_read_04, modify_read_05, modify_read_06, modify_read_07, modify_read_08, 
												modify_read2_01, modify_read2_02, modify_read2_03, modify_read2_04, modify_read2_05, modify_read2_06, modify_read2_07, modify_read2_08, 
												modify_write_01, modify_write_02, modify_write_03, modify_write_04, modify_write_05, 
												modify_inc_01, modify_inc_02, modify_inc_03, modify_inc_04, modify_inc_05
										 );

	signal gme_state					: gme_states;

	signal gme_trigger_z1			: std_logic;

	-- data & address routing
	signal gme_adr_route			: std_logic_vector (1 downto 0);
	signal gme_data_modify		: std_logic_vector (1 downto 0);

	signal s_iomem_rdata 			: std_logic_vector (7 downto 0);	


	signal gme_operation			: std_logic_vector (7 downto 0);

	signal gme_din_reg  			: std_logic_vector (7 downto 0); -- fr transparent


	-- boundary - checks
	signal gme_xd_cnt_end  		: std_logic;
	signal gme_yd_cnt_end  		: std_logic;
	signal gme_r_cnt_end  		: std_logic;

	-- adressen
	signal gme_start_addr_out : std_logic_vector (19 downto 0); 

  signal gme_addr1_base 		: std_logic_vector (19 downto 0);
  signal gme_addr2_base 		: std_logic_vector (19 downto 0);
  signal gme_addr3_base 		: std_logic_vector (19 downto 0);

	signal gme_x_cnt_dst 			: std_logic_vector ( 9 downto 0);
	signal gme_y_cnt_dst 			: std_logic_vector ( 9 downto 0);

	signal gme_width_reg1 		: std_logic_vector ( 9 downto 0);
	signal gme_width_reg2  		: std_logic_vector ( 9 downto 0);

	signal gme_x_size_dst 		: std_logic_vector ( 9 downto 0);
	signal gme_y_size_dst 		: std_logic_vector ( 9 downto 0);

	--signal gme_count_xs_dir		: std_logic;
	--signal gme_count_ys_dir		: std_logic;

	--signal gme_count_xd_dir		: std_logic;
	--signal gme_count_yd_dir		: std_logic;

  signal gme_count_ram 			: std_logic_vector (19 downto 0);
  signal gme_count_ram_max	: std_logic_vector (19 downto 0);

	signal gme_fvalue1				: std_logic_vector ( 7 downto 0); -- data fr replace und mask ...
	signal gme_fvalue2				: std_logic_vector ( 7 downto 0);
	signal gme_fvalue3				: std_logic_vector ( 7 downto 0);

  signal gme_mpos_x   			: std_logic_vector ( 9 downto 0);
  signal gme_mpos_y   			: std_logic_vector ( 9 downto 0);
	signal gme_mouseidx 			: std_logic_vector ( 2 downto 0);
	signal gme_mouse_on   		: std_logic;

	signal gme_palidx 				: std_logic_vector ( 1 downto 0);
  
	signal gme_adr_cfg 				: std_logic_vector ( 3 downto 0);

	signal gme_copy_dir    		: std_logic;

	signal gme_copyrdata			: std_logic;



 	signal gme_busy						: std_logic;

 	signal gme_trigger				: std_logic;
 	signal gme_videooff				: std_logic;

	signal gme_count_xd_res		: std_logic;
	signal gme_count_yd_res		: std_logic;
	signal gme_count_r_res		: std_logic;

	signal gme_count_xd_inc 	: std_logic;
	signal gme_count_yd_inc		: std_logic;
	signal gme_count_r_inc 		: std_logic;

	signal gme_channel     		: std_logic;
	
  constant gme_max_x_size 	: std_logic_vector (9 downto 0) := "1010000000";  -- 280h = 640 

begin


	-- signalausgabe
	
	channel 		<= gme_channel;

	iomem_rdata	<= s_iomem_rdata;

	videooff 		<= gme_videooff;
	videostart	<= gme_start_addr_out;	

	pal_idx			<= gme_palidx;	
	
	mpos_x			<= gme_mpos_x;
	mpos_y			<= gme_mpos_y;
	mouse_on		<= gme_mouse_on;
	mouseidx		<= gme_mouseidx;



	-- io-mem-register setzen
  process (clk, res, iomem_en, iomem_wen, iomem_adr, iomem_wdata)
	begin
		if res = '1' then
			gme_videooff 					<= '0';
			gme_trigger						<= '0';
			gme_operation					<= (others => '0');
			gme_start_addr_out		<= (others => '0');

			gme_addr1_base				<= (others => '0');
			gme_addr2_base				<= (others => '0');
			gme_addr3_base				<= (others => '0');
			

			gme_x_size_dst				<= (others => '0');
			gme_y_size_dst				<= (others => '0');

			gme_count_ram_max			<= (others => '0');

			gme_width_reg1				<=  "1010000000";
			gme_width_reg2				<= (others => '0');


			gme_fvalue1						<= (others => '0');
			gme_fvalue2						<= (others => '0');
			gme_fvalue3						<= (others => '0');

			gme_mpos_x 						<= (others => '0');
			gme_mpos_y 						<= (others => '0');
			gme_mouseidx					<= (others => '0');
			gme_mouse_on					<= '0';

			gme_palidx						<= (others => '0');

			gme_adr_cfg						<= "0010";


			--gme_count_xs_dir			<= '0';
			--gme_count_ys_dir			<= '0';

			--gme_count_xd_dir			<= '0';
			--gme_count_yd_dir			<= '0';


		  gme_channel 			 		<= '0';

			gme_copy_dir					<= '0';

			s_iomem_rdata				  <= (others => '0');


		elsif clk'event and clk = '1' then

		  if iomem_en = '1' then
			  if iomem_wen = '1' then 
				  case iomem_adr is
						-- basic control (video-off, video-start, trigger, operation)
						when X"00" => gme_videooff			<= iomem_wdata (0);
													gme_mouse_on			<= iomem_wdata (1);
						when X"01" => gme_operation 		<= iomem_wdata;
						when X"02" => gme_channel				<= iomem_wdata (0);
						              gme_copy_dir			<= iomem_wdata (1);
													gme_adr_cfg				<= iomem_wdata (7 downto 4);
													--gme_count_xs_dir	<= iomem_wdata (4);
													--gme_count_ys_dir	<= iomem_wdata (5);
													--gme_count_xd_dir	<= iomem_wdata (6);
													--gme_count_yd_dir	<= iomem_wdata (7);
						when X"03" => gme_trigger	 			<= iomem_wdata (0);
						when X"04" => gme_trigger	 			<= iomem_wdata (0);	 -- ntzich bei autoincrement (lsst sich gme_trigger schnell rcksetzen)

						when X"08" => gme_palidx				<= iomem_wdata (1 downto 0);
						when X"09" => gme_mouseidx			<= iomem_wdata (2 downto 0);

						when X"0c" => gme_start_addr_out 	( 7 downto  0) <= iomem_wdata;
						when X"0d" => gme_start_addr_out 	(15 downto  8) <= iomem_wdata;
						when X"0e" => gme_start_addr_out 	(19 downto 16) <= iomem_wdata (3 downto 0);

						-- address start register fr copy/fill/modify operationen
						when X"10" => gme_addr1_base			( 7 downto  0) <= iomem_wdata;
						when X"11" => gme_addr1_base			(15 downto  8) <= iomem_wdata;
						when X"12" => gme_addr1_base			(19 downto 16) <= iomem_wdata (3 downto 0);

						when X"14" => gme_addr2_base			( 7 downto  0) <= iomem_wdata;
						when X"15" => gme_addr2_base			(15 downto  8) <= iomem_wdata;
						when X"16" => gme_addr2_base			(19 downto 16) <= iomem_wdata (3 downto 0);

						when X"18" => gme_addr3_base 			( 7 downto  0) <= iomem_wdata;
						when X"19" => gme_addr3_base 			(15 downto  8) <= iomem_wdata;
						when X"1a" => gme_addr3_base			(19 downto 16) <= iomem_wdata (3 downto 0);

						-- width register fr raw/block operationen
						when X"20" => gme_width_reg1 			( 7 downto  0) <= iomem_wdata;
						when X"21" => gme_width_reg1 			( 9 downto  8) <= iomem_wdata (1 downto 0);

						when X"22" => gme_width_reg2 			( 7 downto  0) <= iomem_wdata;
						when X"23" => gme_width_reg2 			( 9 downto  8) <= iomem_wdata (1 downto 0);

						when X"24" => gme_x_size_dst 			( 7 downto  0) <= iomem_wdata;
						when X"25" => gme_x_size_dst 			( 9 downto  8) <= iomem_wdata (1 downto 0);

						when X"26" => gme_y_size_dst 			( 7 downto  0) <= iomem_wdata;
						when X"27" => gme_y_size_dst 			( 9 downto  8) <= iomem_wdata (1 downto 0);


						when X"28" => gme_count_ram_max		( 7 downto  0) <= iomem_wdata;
						when X"29" => gme_count_ram_max		(15 downto  8) <= iomem_wdata;
						when X"2a" => gme_count_ram_max		(19 downto 16) <= iomem_wdata (3 downto 0);

						-- werte register fr modify
						when X"30" => gme_fvalue1												 <= iomem_wdata;
						when X"31" => gme_fvalue2												 <= iomem_wdata;
						when X"32" => gme_fvalue3												 <= iomem_wdata;


						-- werte register fr mpos
						when X"40" => gme_mpos_x (7 downto 0) 					 <= iomem_wdata;
						when X"41" => gme_mpos_x (9 downto 8) 					 <= iomem_wdata (1 downto 0);
						when X"42" => gme_mpos_y (7 downto 0) 					 <= iomem_wdata;
						when X"43" => gme_mpos_y (9 downto 8) 					 <= iomem_wdata (1 downto 0);


						when others => NULL;
					end case;
				else
				  case iomem_adr is
						-- basic control (video-off, video-start, operation)
						when X"01" => s_iomem_rdata (7 downto 1)	<= (others => '0');
													s_iomem_rdata (0)						<= gme_busy;

						when X"02" => s_iomem_rdata 							<= gme_operation;

						when others => NULL;
					end case;
				end if;
			end if;
		end if;

	end process;






	-- adress berechnung & routing
	process (	gme_adr_route, gme_addr1_base, gme_addr2_base, gme_addr3_base, 
					 	gme_y_cnt_dst, gme_x_cnt_dst, gme_width_reg1, gme_width_reg2,
						gme_count_ram)
	begin
		case gme_adr_route is
		  when "00" 	=> gme_adr <= gme_addr1_base + gme_width_reg1  * gme_y_cnt_dst + gme_x_cnt_dst;
		  when "01" 	=> gme_adr <= gme_addr2_base + gme_width_reg2 * gme_y_cnt_dst + gme_x_cnt_dst;
		  when "10" 	=> gme_adr <= gme_addr3_base + gme_count_ram;
			when others => gme_adr <= (others => '0');
		end case;
	end process;

	-- data modify
	process (gme_data_modify, gme_fvalue1, gme_fvalue2, gme_fvalue3, gme_din, gme_din_reg)
	begin
		case gme_data_modify is
			when "00" 	=> gme_dout <= gme_fvalue1;
			when "01" 	=> gme_dout <= ((gme_din and gme_fvalue1) or gme_fvalue2) xor gme_fvalue3;
			when "10" 	=> if gme_din = X"00" then 
											 gme_dout <= gme_fvalue1;
										 elsif gme_din = X"01" then
											 gme_dout <= gme_fvalue2;
										 else	
											 gme_dout <= gme_din_reg;
										 end if;	
			when "11" 	=> if gme_din = X"00" then 
											 gme_dout <= gme_fvalue1;
										 else	
											 gme_dout <= gme_din_reg;
										 end if;	
			when others => gme_dout <= (others => '0');
		end case;
	end process;


	-- zhler check
	process (--gme_x_cnt_src,  gme_y_cnt_src, gme_y_size_src, gme_width_reg2,
					 gme_x_cnt_dst, gme_x_size_dst, gme_y_cnt_dst, gme_y_size_dst,
					 gme_count_ram, gme_count_ram_max)
	begin
		--if gme_x_cnt_src >= gme_width_reg2 then
		--  gme_xs_cnt_end <= '1';
		--else
		--  gme_xs_cnt_end <= '0';
		--end if;
									              

		--if gme_y_cnt_src >= gme_y_size_src then
		--  gme_ys_cnt_end <= '1';
		--else
		--  gme_ys_cnt_end <= '0';
		--end if;


		if gme_x_cnt_dst >= gme_x_size_dst then
		  gme_xd_cnt_end <= '1';
		else
		  gme_xd_cnt_end <= '0';
		end if;
									              

		if gme_y_cnt_dst >= gme_y_size_dst then
		  gme_yd_cnt_end <= '1';
		else
		  gme_yd_cnt_end <= '0';
		end if;


		if gme_count_ram >= gme_count_ram_max then
		  gme_r_cnt_end <= '1';
		else
		  gme_r_cnt_end <= '0';
		end if;

	end process;

	-- zhler increment/reset
	process (clk, res, --gme_count_xs_res, gme_count_ys_res, 
										 gme_count_xd_res, gme_count_yd_res, gme_count_r_res,
										 --gme_count_xs_inc, gme_count_ys_inc, 
										 gme_count_xd_inc, gme_count_yd_inc, gme_count_r_inc,
										 --gme_count_xs_dir, gme_count_ys_dir, gme_count_xd_dir, gme_count_yd_dir,
										 --gme_x_cnt_src, gme_y_cnt_src, 
										 gme_x_cnt_dst, gme_y_cnt_dst, gme_count_ram)
	begin
	 if res = '1' then
		 --gme_x_cnt_src <= (others => '0');
		 --gme_y_cnt_src <= (others => '0');
		 gme_x_cnt_dst <= (others => '0');
		 gme_y_cnt_dst <= (others => '0');
		 gme_count_ram <= (others => '0');


	 elsif clk'event and clk = '1' then

		  --if gme_count_xs_res = '1' then
			  --if gme_count_xs_dir = '0' then
			--  	gme_x_cnt_src <= (others => '0');
				--else
				--	gme_x_cnt_src <= gme_width_reg2;
				--end if;
			--end if;

		  --if gme_count_ys_res = '1' then
			  --if gme_count_ys_dir = '0' then
				--	gme_y_cnt_src <= (others => '0');
				--else
				--	gme_y_cnt_src <= gme_y_size_src;
				--end if;
			--end if;

		  if gme_count_xd_res = '1' then
			  --if gme_count_xd_dir = '0' then
			  	gme_x_cnt_dst <= (others => '0');
				--else
			  --	gme_x_cnt_dst <= gme_x_size_dst;
				--end if;
			end if;

		  if gme_count_yd_res = '1' then
			  --if gme_count_yd_dir = '0' then
					gme_y_cnt_dst <= (others => '0');
				--else
					--gme_y_cnt_dst <= gme_y_size_dst;
				--end if;
			end if;

		  if gme_count_r_res = '1' then
			  gme_count_ram <= (others => '0');
			end if;



		  --if gme_count_xs_inc = '1' then
			  --if gme_count_xs_dir = '0' then
				--	gme_x_cnt_src <= gme_x_cnt_src + '1';
				--else
				--	gme_x_cnt_src <= gme_x_cnt_src - '1';
				--end if;
			--end if;

		  --if gme_count_ys_inc = '1' then
			  --if gme_count_ys_dir = '0' then
				--		gme_y_cnt_src <= gme_y_cnt_src + '1';
				--else
				--	gme_y_cnt_src <= gme_y_cnt_src - '1';
				--end if;
			--end if;

		  if gme_count_xd_inc = '1' then
			  --if gme_count_xd_dir = '0' then
						gme_x_cnt_dst <= gme_x_cnt_dst + '1';
				--else
				--	gme_x_cnt_dst <= gme_x_cnt_dst - '1';
				--end if;
			end if;

		  if gme_count_yd_inc = '1' then
			  --if gme_count_yd_dir = '0' then	
						gme_y_cnt_dst <= gme_y_cnt_dst + '1';
				--else
				--	gme_y_cnt_dst <= gme_y_cnt_dst - '1';
				--end if;
			end if;


		  if gme_count_r_inc = '1' then
				gme_count_ram <= gme_count_ram + '1';
			end if;

	 end if;

	end process;



	-- dp-statemachine
  process (clk, res)
	begin
		if res = '1' then
			gme_state 		 		<= idle;
			gme_trg				 		<= '0';
			gme_trigger_z1 		<= '1';
	  	gme_busy 					<= '0';
			gme_rw						<= '0';

			--gme_count_xs_res	<= '0';
			--gme_count_ys_res	<= '0';
			gme_count_xd_res	<= '0';
			gme_count_yd_res	<= '0';
			gme_count_r_res 	<= '0';


			--gme_count_xs_inc	<= '0';
			--gme_count_ys_inc	<= '0';
			gme_count_xd_inc	<= '0';
			gme_count_yd_inc	<= '0';
			gme_count_r_inc 	<= '0';

			gme_data_modify				<= "00";
			gme_adr_route					<= "00";

			gme_copyrdata			<= '0';

			gme_din_reg				<= (others => '0');


		elsif clk'event and clk = '1' then
			
			if gme_state = idle then
			  gme_busy <= '0';
			else
			  gme_busy <= '1';
			end if;


			if gme_copyrdata = '1' then
				gme_din_reg <= gme_din;
			end if;

			gme_trigger_z1 <= gme_trigger;

			case gme_state is

				-- hier wird auf gme_trigger gewartet und wenns denn da ist in den jeweilgen state (nach gme_oepration)
				when idle =>
					if gme_trigger_z1 = '0' and gme_trigger = '1' then
						case gme_operation is
							when X"01" =>	gme_state <= fill_setup_01;
							
							when X"04" =>	gme_data_modify <= "00"; gme_state <= modify_setup_01;
							when X"05" =>	gme_data_modify <= "01"; gme_state <= modify_setup_01;
							when X"06" =>	gme_data_modify <= "10"; gme_state <= modify_setup_01;

				 			when others => gme_state <= idle;
						end case;							
					end if;

				when fill_setup_01	=>
					gme_count_r_res <= '1';	

					gme_data_modify			<= "00";
					gme_adr_route				<= "10";

					gme_rw					<= '0';	  

					gme_state 			<= fill_setup_02;

				when fill_setup_02	=>
					gme_count_r_res <= '0';	
					gme_state 			<= fill_write_01;


				when fill_write_01		=>
					if gme_req = '1' then
					  gme_trg					<= '1';
					  gme_state 			<= fill_write_02;
					else	 
					  gme_state 			<= fill_write_01;
					end if;

				when fill_write_02		=>
				  gme_trg					<= '1';
				  gme_state 			<= fill_write_03;

				when fill_write_03		=>
				  gme_trg					<= '0';
				  gme_state 			<= fill_write_04;

				when fill_write_04		=>
				  gme_count_r_inc <= '0';
				  gme_state 			<= fill_inc_01;

				when fill_inc_01		=>
				  gme_count_r_inc <= '1';
					gme_state 			<= fill_inc_02;

				when fill_inc_02		=>
				  gme_count_r_inc <= '0';
					gme_state 			<= fill_inc_03;

				when fill_inc_03		=>
					if gme_r_cnt_end = '1' then
						gme_state 			<= idle;
					else
						gme_state				<= fill_write_01;
					end if;














				when modify_setup_01	=>
					gme_count_xd_res 	<= '1';	
					gme_count_yd_res 	<= '1';	
					gme_count_r_res 	<= '1';	

					gme_state 				<= modify_setup_02;

				when modify_setup_02	=>
					gme_count_xd_res 	<= '0';	
					gme_count_yd_res 	<= '0';	
					gme_count_r_res 	<= '0';	

					case gme_data_modify is
					  when "10" 	=> gme_state <= modify_read_01;
						when "01" 	=> gme_state <= modify_read_08;
						when "00" 	=> gme_state <= modify_read2_01;
						when others => NULL;
					end case;





				when modify_read_01		=>
					gme_adr_route			<= "00";
					gme_state 				<= modify_read_02;

				when modify_read_02		=>
					if gme_req = '1' then
					  gme_trg					<= '1';
						gme_rw					<= '1';	  
					  gme_state 			<= modify_read_03;
					else	 
					  gme_state 			<= modify_read_02;
					end if;

				when modify_read_03		=>
				  gme_trg						<= '1';
				  gme_state 				<= modify_read_04;

				when modify_read_04		=>
				  gme_trg						<= '0';
				  gme_state 				<= modify_read_05;

				when modify_read_05		=>
				  gme_state 		 	 	<= modify_read_06;
				when modify_read_06		=>
				  gme_state 		 	 	<= modify_read_07;
				when modify_read_07		=>
					gme_copyrdata			<= '1';
				  gme_state 		 	 	<= modify_read_08;
				when modify_read_08		=>
					gme_copyrdata			<= '0';
				  if gme_copy_dir = '0' then
						gme_adr_route				<= gme_adr_cfg (1 downto 0);
					else
						gme_adr_route				<= gme_adr_cfg (3 downto 2);
					end if;
				  gme_state 		 	 	<= modify_read2_01;



				when modify_read2_01		=>
					gme_state 			<= modify_read2_02;

				when modify_read2_02		=>
					if gme_req = '1' then
					  gme_trg					<= '1';
						gme_rw					<= '1';	  
					  gme_state 			<= modify_read2_03;
					else	 
					  gme_state 			<= modify_read2_02;
					end if;

				when modify_read2_03		=>
				  gme_trg						<= '1';
				  gme_state 				<= modify_read2_04;

				when modify_read2_04		=>
				  gme_trg						<= '0';
				  gme_state 				<= modify_read2_05;

				when modify_read2_05		=>
				  gme_state 		 	 	<= modify_read2_06;
				when modify_read2_06		=>
				  gme_state 		 	 	<= modify_read2_07;
				when modify_read2_07		=>
				  gme_state 		 	 	<= modify_read2_08;
				when modify_read2_08		=>
				  if gme_copy_dir = '0' then
						gme_adr_route				<= gme_adr_cfg (3 downto 2);
					else
						gme_adr_route				<= gme_adr_cfg (1 downto 0);
					end if;
				  gme_state 		 	 	<= modify_write_01;




				when modify_write_01		=>
					if gme_req = '1' then
					  gme_trg					<= '1';
						gme_rw					<= '0';
					  gme_state 			<= modify_write_02;
					else	 
					  gme_state 			<= modify_write_01;
					end if;		
					
				when modify_write_02		=>
				  gme_trg						<= '1';
				  gme_state 				<= modify_write_03;

				when modify_write_03		=>
				  gme_trg						<= '0';
				  gme_state 				<= modify_write_04;
				when modify_write_04		=>
				  gme_state 				<= modify_write_05;
				when modify_write_05		=>
				  gme_state 				<= modify_inc_01;

				when modify_inc_01		=>
				  gme_count_r_inc  		<= '1';
					gme_count_xd_inc  	<= '1';
				  gme_state 					<= modify_inc_02;

				when modify_inc_02		=>
				  gme_count_r_inc  		<= '0';
					gme_count_xd_inc  	<= '0';
				  gme_state 					<= modify_inc_03;

				when modify_inc_03		=>

					if gme_xd_cnt_end	= '1' then
					  gme_count_yd_inc 	<= '1';
						gme_count_xd_res 	<= '1';
					end if;
				  gme_state 					<= modify_inc_04;


				when modify_inc_04		=>
				  gme_count_yd_inc 		<= '0';
					gme_count_xd_res 		<= '0';
				  gme_state 					<= modify_inc_05;

				when modify_inc_05		=>
				  if gme_yd_cnt_end = '1'	then
				  	gme_state 				<= idle;
					else
						case gme_data_modify is
						  when "10" 	=> gme_state <= modify_read_01;
							when "01" 	=> gme_state <= modify_read_08;
							when "00" 	=> gme_state <= modify_read2_01;
							when others => NULL;
						end case;
					  --gme_state					<= modify_read_01;
					end if;

				when others => NULL;	  
			end case;




		end if;

	end process;


end Behavioral;
